/***************************************************************************
                          cmdlineoptions.cpp  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Andr�Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmdlineoptions.h"

using namespace std;
using namespace astyle;

/* Siehe man getopt (3)
   Konstruktor legt Optionen und Argumente fest
*/
CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
    numberSpaces(0),
    wrappingStyle(0),
    formatStyle(DISABLED),
    cssOutFilename("highlight.css"),    
    opt_outfile (false),
    opt_infile (false),
    opt_language (false),
    opt_include_css (false),
    opt_help (false),
    opt_version (false),
    opt_verbose (false),
    opt_linenumbers (false),
    opt_css_style (false),
    opt_css_outfilename (false),
    opt_css_infilename (false),  
    opt_xhtml_output (false),
    opt_batch_mode (false),
    opt_batch_recursive(false),
    opt_rtf_output (false),
    opt_tex_output (false),
    opt_latex_output (false),
    opt_xslfo_output (false),
    opt_fragment (false) ,
    opt_attach_line_anchors (false),
    opt_show_themes (false),
    opt_show_langdefs (false),
    opt_outdir(false),
    opt_asformat_output(false),
    opt_data_dir(false),
    opt_printindex(false),
    opt_quiet(false),
    opt_xslfo_fop(false),
    helpLang("en")
{

  loadPresets();

#ifndef USE_PORTABLE_CMDLINE_PARSER

  int c, option_index = 0;

  static struct option long_options[] =
      {
        {"output", 1, 0, 'o'},
        {"input", 1, 0, 'i'},
        {"syntax", 1, 0, 'S'},
        {"verbose", 0, 0, 'v'},
        {"includecss", 0, 0, 'I'},
        {"help", 0, 0, 'h'},
        {"help-int", 1, 0, 'H'},
        {"linenumbers",0,0,'l'},
        {"style", 1,0,'s'},
        {"css-outfile", 1, 0,'c'},
        {"css-infile", 1, 0,'e'},
        {"deletetabs",1,0,'t'},
        {"hilfe", 0,0,'d'},        
        {"xhtml", 0,0,'X'},
        {"rtf", 0,0,'R'},
        {"tex",0, 0,'T'},
        {"latex",0, 0,'L'},
        {"xsl-fo",0, 0,'Y'},
        {"batch",1,0,'b'},
        {"batch-recursive",1,0,'B'},        
        {"fragment",0,0,'f'},
        {"anchors", 0,0,'a' },
        {"listthemes", 0,0,'w' },
        {"listlangs", 0,0,'p' },
        {"outdir",1,0,'O'},
        {"version",0,0,0},
        {"format-style",1,0,'F'},
        {"data-dir",1,0,'D'},
        {"printindex",0,0,'C'},
        {"wrap",0,0,'W'},
        {"wrap-simple",0,0,'V'},
        {"quiet",0,0,'q'},
        //remove as son as APAche fixed the bug in FOP (0.20.5)
        {"fop-compatible",0,0,'A'}, 
        {0, 0, 0, 0}
      };
#else
	CommandLineParser p;
	p.addSwitchOption('v', "verbose");	
        p.addSwitchOption('q', "quiet");	
	p.addSwitchOption('I', "includecss");
	p.addSwitchOption('h', "help");
	p.addSwitchOption('d', "hilfe");
	p.addSwitchOption('l', "linenumbers");
	p.addSwitchOption('X', "xhtml");
	p.addSwitchOption('R', "rtf");      
	p.addSwitchOption('T', "tex");      
	p.addSwitchOption('L', "latex");      
	p.addSwitchOption('Y', "xsl-fo");
	p.addSwitchOption('f', "fragment");        	
	p.addSwitchOption('a', "anchors");      
	p.addSwitchOption('w', "listthemes");
	p.addSwitchOption('p', "listlangs");      
	p.addSwitchOption('Q', "version");      
	p.addStringOption('i', "input");	
	p.addStringOption('o', "output");	
	p.addStringOption('S', "syntax");	
	p.addStringOption('s', "style");	
	p.addStringOption('c', "css-outfile");	
	p.addStringOption('e', "css-infile");	
	p.addIntOption('t', "deletetabs");	
	p.addStringOption('b', "batch");	
	p.addStringOption('B', "batch-recursive");	
	p.addStringOption('O', "outdir");	
	p.addStringOption('F', "format-style");	
	p.addStringOption('D', "data-dir");
	p.addStringOption('C', "printindex");	
	p.addStringOption('H', "help-int");	
	p.addSwitchOption('W', "wrap");	
	p.addSwitchOption('V', "wrap-simple");
       //remove as son as APAche fixed the bug in FOP (0.20.5)
        p.addSwitchOption('A', "fop-compatible");	
	try
	{
		p.parse(argc, argv);	
	}
	// you need to enable exceptions within your compiler settings
	catch (ArgumentParseException& ex)
	{
		cerr << ex.what() << endl;
	}
#endif      

#ifndef USE_PORTABLE_CMDLINE_PARSER
  while (1)
    {
      c = getopt_long (argc, argv,
                       "o:i:S:b:B:O:adfhlvwpqILYRTXs:c:e:t:F:D:CH:VWA",
                       long_options, &option_index);
      if (c == -1)
        break;

      switch (c)
        {
        case 0:   //nur long options
          if (long_options[option_index].name=="version")
            {
              opt_version = true;
            }
          break;
        case 'o':
          outFilename=string(optarg);
          opt_outfile = true;
          break;
        case 'i':          
          opt_infile = true;
          inputFileNames.push_back(string(optarg));  
          break;
        case 'c':
          cssOutFilename=string(optarg);
          opt_css_outfilename = true;
          break;
        case 'e':
          cssInFilename=string(optarg);
          opt_css_infilename = true;
          break;
        case 'v':
          opt_verbose = true;
          break;
        case 'q':
          opt_quiet = true;
          break;
        case 'I':
          opt_include_css = true;
          break;
        case 'd':
          helpLang="de";
          opt_help = true;
          break;
        case 'H':
          helpLang=string(optarg);
        case 'h':        
          opt_help = true;
          break;
        case 'l':
          opt_linenumbers = true;
          break;
        case '?':
          opt_help = true;
          break;
        case 's':
          styleName=string(optarg);
          opt_css_style = true;
          break;
        case 'S':
          language=string(optarg);
          opt_language = true;
          break;
        case 't':         
          numberSpaces = StringTools::str2int (optarg);
          break;
        case 'X':
          opt_xhtml_output = true;
          break;
        case 'R':
          opt_rtf_output = true;
          break;
        case 'T':
          opt_tex_output = true;
          break;
        case 'L':
          opt_latex_output = true;
          break;
        case 'Y':
          opt_xslfo_output = true;
          break;
        case 'B':
          opt_batch_recursive = true;  
        case 'b':          
          if (!language.size())
            language= string(optarg);
          opt_batch_mode = true;
          readDirectory(string(optarg), opt_batch_recursive);
          break;
        case 'f':
          opt_fragment = true;
          break;
        case 'a':
          opt_attach_line_anchors = true;
          break;
        case 'w':
          opt_show_themes = true;
          break;
        case 'p':
          opt_show_langdefs = true;
          break;
        case 'O':
          outDirectory = string(optarg);
          if (outDirectory[outDirectory.length()-1] !='/')
            {
              outDirectory += "/";
            }
          opt_outdir = true;
          break;
        case 'F':
          setFormatStyle(optarg);
          break;
        case 'D':
          dataDir=string(optarg);
          if (dataDir[dataDir.length()-1] !='/')
            {
              dataDir += "/";
            }
          opt_data_dir=true;
          break;
         case 'C':
          opt_printindex=true;
          break;
         case 'V':
          wrappingStyle = 1;
          break;
         case 'W':
          wrappingStyle = 2;
          break;
         case 'A':
          opt_xslfo_fop=true;
          break;
        default:
          cerr <<"?? unknown option " <<c<< endl;
        }
    }
#else

  if (p.hasOption("o")){
          outFilename=p["output"].toString();
          opt_outfile = true;
   }
  if (p.hasOption("i")){
          opt_infile = true;
          inputFileNames.push_back(p["input"].toString());  
   }
   if (p.hasOption("c")){
          cssOutFilename=p["css-outfile"].toString();
          opt_css_outfilename = true;
   }
   if (p.hasOption("e")){
          cssInFilename=p["css-infile"].toString();
          opt_css_infilename = true;
   }   
   if ((bool)p["d"]){
           helpLang="de";
   }
   if (p.hasOption("H")){
          helpLang=p["help-int"].toString();
          opt_help = true;
   }
   if (p.hasOption("s")){
          styleName=p["style"].toString();
          opt_css_style = true;
   }
   if (p.hasOption("S")){
          language=p["syntax"].toString();
          opt_language = true;
   }
   if (p.hasOption("t")){        
          numberSpaces = p["deletetabs"].toInt();
   }   
   if (p.hasOption("B")){
      opt_batch_recursive = true;
      string batchwildcard = (string)p["batch-recursive"];
      if (!language.size())
        language= p["batch-recursive"].toString();
      opt_batch_mode = true;
      readDirectory(batchwildcard, opt_batch_recursive);
  }
  if (p.hasOption("b")){
      string batchwildcard = (string)p["batch"];
      if (!language.size())
        language= p["batch"].toString();
      opt_batch_mode = true;
      readDirectory(batchwildcard, opt_batch_recursive);
   }
   if (p.hasOption("O")){
        outDirectory = p["outdir"].toString();
        if (outDirectory[outDirectory.length()-1] !=PATH_SEPARATOR_CHAR)
            {
              outDirectory += PATH_SEPARATOR_STR;
            }
        opt_outdir = true;
    }           
    if (p.hasOption("F")){
       string f=p["format-style"].toString();
       setFormatStyle(f.c_str());
    }    
    if (p.hasOption("D")){
       dataDir=p["data-dir"].toString();
       if (dataDir[dataDir.length()-1] !=PATH_SEPARATOR_CHAR)
          {
           dataDir += PATH_SEPARATOR_STR;
         }
       opt_data_dir=true;
    }
   if (p["V"]){
     wrappingStyle = 1;
   }
   if (p["W"]){
     wrappingStyle = 2;
   }

   opt_fragment = p["f"];
   opt_attach_line_anchors = p["a"];
   opt_show_themes = p["w"];
   opt_show_langdefs = p["p"];
   opt_printindex=p["C"];
   opt_help = (bool)p["h"] || (bool)p["d"];
   opt_linenumbers = p["l"];
   opt_version = p["Q"];
   opt_verbose = p["v"];
   opt_quiet = p["q"];
   opt_include_css = p["I"];

   opt_xhtml_output = p["X"];
   opt_rtf_output = p["R"];
   opt_tex_output = p["T"];
   opt_latex_output = p["L"];
   opt_xslfo_output = p["Y"];
   opt_xslfo_fop = p["A"];
#endif    

  #ifndef USE_PORTABLE_CMDLINE_PARSER
  if (optind < argc)   //still args left
  #else
  if (argc > 1)
  #endif
    {
      //Inputfile noch unbekannt ?
      if  (inputFileNames.empty())
        {
           #ifndef USE_PORTABLE_CMDLINE_PARSER          
           while (optind < argc){
            inputFileNames.push_back(string(argv[optind++]));  
           }
           #else
           int argIndex=0;              
           while (++argIndex < argc){
            inputFileNames.push_back(string(argv[argIndex]));  
          }          
          #endif                  
          opt_infile=true;
        }      
    }  
}

CmdLineOptions::~CmdLineOptions(){
}

void CmdLineOptions::setFormatStyle(const char * style)
{
  string arg (style);
  opt_asformat_output =  true;

if ( arg =="ansi")
    {
        formatStyle =ANSI;
    }
  else if ( arg =="gnu")
    {
        formatStyle = GNU;
    }
  else if ( arg =="java")
    {
        formatStyle = JAVA;
    }
  else if ( arg =="kr")
    {
        formatStyle = KR;
    }
  else if ( arg =="linux")
    {
        formatStyle = LINUX;
    }
  else
    {
      opt_asformat_output =  false;
      cerr << " highlight: Unknown reformatting argument (-F): "
           << arg
           << "." << endl;
    }  
}

const string &CmdLineOptions::getSingleOutFilename() 
  {
   if (opt_infile && opt_outdir && !opt_outfile){
      int delim;
      delim = getSingleInFilename().find_last_of(PATH_SEPARATOR_CHAR);
      outFilename = outDirectory +
                   getSingleInFilename().substr((delim>-1)?delim:0) 
                   + getOutFileSuffix();
    }
    return outFilename;
  }

const string CmdLineOptions::getSingleInFilename()  const
  {
    return (inputFileNames.size())?inputFileNames[0]:"";
  }

const string &CmdLineOptions::getOutDirectory()const
  {
    return outDirectory;
  }

const string &CmdLineOptions::getCssOutFilename() const
  {
    return cssOutFilename;
  }
const string &CmdLineOptions::getCssInFilename() const
  {
    return cssInFilename;
  }
int CmdLineOptions::getNumberSpaces() const
  {
    return numberSpaces;  
  }
/** gibt true zurck, falls Versionsinfo ausgegeben werden soll */
bool CmdLineOptions::printVersion()const
  {
    return opt_version;
  }
/** gibt true zurck, falls ein Hilfetext ausgegeben werden soll */
bool CmdLineOptions::printHelp()const
  {
    return opt_help;
  }
/** gibt true zurck, falls Debuginfo ausgegeben werden soll */
bool CmdLineOptions::printDebugInfo()const
  {
    return opt_verbose;
  }
/** gibt true zurck, falls keine unnötigen Infos ausgegeben werden sollen */
bool CmdLineOptions::quietMode()const
  {
    return opt_quiet;
  }
/** gibt true zurck, falls CSS in Ausgabedatei eingebettet werden soll */
bool CmdLineOptions::includeCssDefinitions()const
  {
    return opt_include_css;
  }
/** gibt true zurck, falls Zeilennummern ausgegeben werden sollen */
bool CmdLineOptions::printLineNumbers()const
  {
    return opt_linenumbers;
  }
/** gibt true zurck, falls XHTML-Datei ausgegeben werden soll */
bool CmdLineOptions::xhtmlFormat()const
  {
    return opt_xhtml_output;
  }
/** gibt true zurck, falls XSL-FO-Datei ausgegeben werden soll */
bool CmdLineOptions::xslfoFormat()const
  {
    return opt_xslfo_output;
  }
/** gibt true zurck, falls HTML-Datei ausgegeben werden soll */
bool CmdLineOptions::htmlFormat()const
  {
    return !(opt_rtf_output||opt_xhtml_output||opt_latex_output
             || opt_tex_output);
  }
/** gibt true zurck, falls RTF-Datei ausgegeben werden soll */
bool CmdLineOptions::rtfFormat()const
  {
    return opt_rtf_output;
  }
/** gibt true zurck, falls TeX-Datei ausgegeben werden soll */
bool CmdLineOptions::texFormat()const
  {
    return  opt_tex_output;
  }
/** gibt true zurck, falls LaTeX-Datei ausgegeben werden soll */
bool CmdLineOptions::latexFormat()const
  {
    return opt_latex_output;
  }
/** gibt gewuenschten Stil aus (Default: kwrite) */
string CmdLineOptions::getStyleName()const
  {
    return ( ( opt_css_style) ? styleName+".style" : "kwrite.style" );
  }
/** gibt true zurck, falls Batchmodus aktiviert wurde,
    oder falss mehrere Eingabedateien angegeben wurden */
bool CmdLineOptions::enableBatchMode()const
  {
    return inputFileNames.size()>1 || opt_batch_mode;
  }

bool CmdLineOptions::fragmentOutput()const
  {
    return opt_fragment;
  }

/** gibt Dateieindung der Ausgabedatei(en) zurueck */
string CmdLineOptions::getOutFileSuffix()const
  {
    if (opt_xhtml_output)
      return ".xhtml";
    if (opt_rtf_output)
      return ".rtf";
    if (opt_xslfo_output)
      return ".fo";
    if  (opt_tex_output || opt_latex_output)
      return ".tex";
    return ".html";
  }
/** gibt true zurck, falls Anker an Zeilennummerierung gesetzt werden sollen */
bool CmdLineOptions::attachLineAnchors()const
  {
    return opt_attach_line_anchors;
  }

bool CmdLineOptions::showThemes()const
  {
    return opt_show_themes;
  }

bool CmdLineOptions::showLangdefs()const
  {
    return opt_show_langdefs;
  }

bool CmdLineOptions::outDirGiven()const
  {
    return opt_outdir;
  }

bool CmdLineOptions::fopCompatible() const {
   return opt_xslfo_fop;
}

void CmdLineOptions::loadPresets()
{
 
 string paramVal;
  #ifndef _WIN32
  string path=getenv("HOME");  
  path += "/.highlight/highlight.conf";
  #else
  string path = "highlight.conf";
  #endif  
  ConfigurationReader presets(path, false);
  string trueStr="true";
  if (presets.found())
    { 
      if (printDebugInfo()) {
        clog << "Reading configuration file "
             << path << endl;
      }
      paramVal = presets.getParameter("style");
      if (!paramVal.empty())
        {
          styleName=paramVal;
          opt_css_style = true;         
        }
      paramVal = presets.getParameter("deletetabs");
      if (!paramVal.empty())
        {          
          numberSpaces = StringTools::str2int (paramVal );         
        }
      paramVal = presets.getParameter("linenumbers");      
      if (paramVal == trueStr)
        {
          opt_linenumbers = true;
        }
      paramVal = presets.getParameter("includecss");   
      if (paramVal == trueStr)
        {
          opt_include_css = true;
        }
      paramVal = presets.getParameter("wrap");   
      if (paramVal == trueStr)
        {
          wrappingStyle=2;
        }
      paramVal = presets.getParameter("wrapsimple");   
      if (paramVal == trueStr)
        {
          wrappingStyle=1;
        }
      paramVal = presets.getParameter("css-outfile");
      if (!paramVal.empty())
        {
          cssOutFilename=paramVal;
          opt_css_outfilename = true;
        }
      paramVal = presets.getParameter("css-infile");
      if (!paramVal.empty())
        {
          cssInFilename=paramVal;
          opt_css_infilename = true;
        }
      paramVal = presets.getParameter("format-style");
      if (!paramVal.empty())
        {
          setFormatStyle(paramVal.c_str());         
        }
      paramVal = presets.getParameter("data-dir");
      if (!paramVal.empty()) 
        {
          dataDir=paramVal;
          opt_data_dir=true;          
        }
    }
}


bool CmdLineOptions::formattingEnabled() 
  {/*
    if (opt_asformat_output)
    {
      if (   (language=="c") ||(language=="cs")
           ||(language=="java"))
        {
          numberSpaces = 0;  // Tabs werden durch ASFormatter ersetzt
        }
      else
        {
          cerr << "highlight: Code reformatting only enabled with "
               << "C, C++, C# or Java input files."
               << endl;
          opt_asformat_output=false;
        }
    }*/
    return opt_asformat_output;
  }

bool CmdLineOptions::newDataDir()const {
  return opt_data_dir;  
}

/** gibt Datenverzeichnis zurck */
string  CmdLineOptions::getDataDir() const {
 return dataDir;
}

/** gibt Datenverzeichnis zurck */
string  CmdLineOptions::getLanguage()  {
 return language;
}
/** true, falls Verzeichnisse rekursiv durchsucht werden sollen */
bool CmdLineOptions::batchRecursive() const{
 return opt_batch_recursive;
}

bool CmdLineOptions::printIndexFile() const{
 return opt_printindex && (htmlFormat() || xhtmlFormat());
}

bool CmdLineOptions::syntaxGiven() const{
  return opt_language;
}

/** No descriptions */
enum FormatStyle CmdLineOptions::getFormatStyle(){
  return formatStyle;
}

const string & CmdLineOptions::getHelpLang()const{
  return helpLang;
}

int CmdLineOptions::getWrappingStyle() const {
  return wrappingStyle;
}

const vector <string> & CmdLineOptions::getInputFileNames() const{
  return inputFileNames;
}

void CmdLineOptions::readDirectory(const string & wildcard, bool recursive){
  bool directoryOK=DirectoryReader::getEntries (inputFileNames,
                                                wildcard,
                                                recursive);
  if (!directoryOK)
    {
       cerr << "highlight: No files matched the pattern "
            << wildcard << "."<< endl;      
    }      
}
