/***************************************************************************
                          cmdlineoptions.h  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Andr�Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CMDLINEOPTIONS_H
#define CMDLINEOPTIONS_H

#include <string>
#include <map>
#include <stdlib.h>
#include <iostream>
#include <fstream>

#include "stringtools.h"
#include "configurationreader.h"
#include "datadir.h"
#include "ASFormatter.h"
#include "directoryreader.h"

// If your system does not know getopt_long, define USE_PORTABLE_CMDLINE_PARSER
#if defined(_WIN32) || defined(__SVR4) || defined(__sun__) 
  #define USE_PORTABLE_CMDLINE_PARSER
#endif  

#ifndef USE_PORTABLE_CMDLINE_PARSER
#include <getopt.h>
#else
#include "win32cmdline.h"
#endif


#define L_OPT_VERBOSE  "verbose"
#define L_OPT_INCLUDECSS  "includecss"
#define L_OPT_HELP  "help"
#define L_OPT_HILFE  "hilfe"
#define L_OPT_LINENO  "linenumbers"
#define L_OPT_XHTML  "xhtml"
#define L_OPT_RTF  "rtf"      
#define L_OPT_TEX  "tex"      
#define L_OPT_LATEX  "latex"      
#define L_OPT_XSLFO  "xsl-fo"
#define L_OPT_FRAGMENT  "fragment" 
#define L_OPT_ANCHORS  "anchors"      
#define L_OPT_LISTTH  "listthemes"
#define L_OPT_LISTLA  "listlangs"      
#define L_OPT_VERSION  "version"      
#define L_OPT_IN  "input"
#define L_OPT_OUT  "output"
#define L_OPT_SYNTAX  "syntax"
#define L_OPT_STYLE "style"
#define L_OPT_CSSOUT  "css-outfile"
#define L_OPT_CSSIN  "css-infile"
#define L_OPT_DELTABS  "deletetabs"
#define L_OPT_BATCH  "batch"
#define L_OPT_BATCHR  "batch-recursive"
#define L_OPT_OUTDIR  "outdir"
#define L_OPT_FORMATSTYLE  "format-style"
#define L_OPT_DATADIR  "data-dir"
#define L_OPT_INDEXFILE  "printindex"
#define L_OPT_HELPINT  "help-int"
#define L_OPT_WRAP  "wrap"
#define L_OPT_WRAPSIMPLE  "wrap-simple"

using namespace std;
 
class CmdLineOptions
  {
  public:

    //Konstruktor erhaelt Kommandozeilenparameter
    CmdLineOptions(int argc, char *argv[]);
    ~CmdLineOptions();
    
    /** gibt Namen der Ausgabedatei zurueck*/
    const string &getSingleOutFilename();
    /** gibt Namen der Eingabedatei zurueck*/
    const string getSingleInFilename() const;
    /** gibt Namen des Ausgabeverzeichnisses zurueck*/
    const string&getOutDirectory() const;
    
    /** gibt Namen der CSS-Ausgabedatei zurueck*/
    const string&getCssOutFilename() const;
    /** gibt Namen der CSS-Eingabedatei zurueck*/
    const string&getCssInFilename() const;
    
    /** gibt Anzahl der Leerzeichen zurueck, die Tabs ersetzen sollen*/
    int getNumberSpaces() const;
    /** gibt true zurck, falls Versionsinfo ausgegeben werden soll */
    bool printVersion() const;
    /** gibt true zurck, falls ein Hilfetext ausgegeben werden soll */
    bool printHelp() const;
    /** gibt true zurck, falls Debuginfo ausgegeben werden soll */
    bool printDebugInfo()const;
    /** gibt true zurck, falls CSS in Ausgabedatei eingebettet werden soll */
    bool includeCssDefinitions() const;
    /** gibt true zurck, falls Zeilennummern ausgegeben werden sollen */
    bool printLineNumbers() const;
    /** gibt gewuenschten Themenamen aus */
    string getStyleName()const ;

    /** gibt true zurck, falls bestimmtes Format ausgegeben werden soll */
    bool xhtmlFormat() const;
    bool rtfFormat() const;
    bool texFormat() const;
    bool latexFormat() const;     
    bool htmlFormat() const;
    bool xslfoFormat() const;

    /** gibt true zurck, falls deutsche Hilfe ausgegeben werden soll */
    int helpLanguage() const;
    
    /** gibt true zurck, falls Batchmodus aktiviert wurde */
    bool enableBatchMode() const;
    /** gibt true zurck, falls Header und Footer-Teil der Ausgabedatei
        weggelassen werden sollen */
    bool fragmentOutput() const;

    /** gibt Dateieindung der Ausgabedatei(en) zurueck */
    string getOutFileSuffix() const;
    /** gibt true zurck, falls Anker an Zeilennummerierung gesetzt werden sollen */
    bool attachLineAnchors() const;
    /** gibt true zurck, falls installierte Style-Themes angezeigt werden sollen */
    bool showThemes() const;
    /** gibt true zurck, falls installierte lang-Dateien angezeigt werden sollen */
    bool showLangdefs() const;

    /** gibt true zurck, falls Ausgabeverzeichnis gesetzt wurde */
    bool outDirGiven() const;

    bool formattingEnabled();

    bool newDataDir()const;
    bool printIndexFile() const;
      
  /** gibt Datenverzeichnis zurck */
  string  getDataDir()const;
  /** true, falls Verzeichnisse rekursiv durchsucht werden sollen */
  bool batchRecursive() const;
  
  bool syntaxGiven() const;

  bool quietMode() const;
  bool fopCompatible() const;
  /** No descriptions */
  enum FormatStyle getFormatStyle();

  const string & getHelpLang() const;
  string getLanguage() ;
  
  int getWrappingStyle() const;
 
  const vector <string> & getInputFileNames() const;

  private:

    int numberSpaces;   //anzahl Spaces, die einen Tabulator ersetzen sollen
    int wrappingStyle;
    
    enum FormatStyle formatStyle;
 //   string errmsg;  //Fehlermeldung
    
    //Name der Ausgabedatei (falls kein Batch-Modus aktiviert ist)
    string outFilename, 
    outDirectory,   //Zielverzeichnis fuer Ausgabedateien
    language,       //Programmiersprache, die geparst wird
    styleName,      //Stil der formatierung (zB kwrite oder emacs)
    cssOutFilename, //Name der CSS-Ausgabedatei
    cssInFilename,  //Name der CSS-Eingabedatei
    dataDir;        //Datenverzeichnis

    //Flags zur Bestimmung der gesetzten Parameter:
    bool opt_outfile;
    bool opt_infile;
    bool opt_language;
    bool opt_include_css;
    bool opt_help;
    bool opt_version ;
    bool opt_verbose;
    bool opt_linenumbers;
    bool opt_css_style;
    bool opt_css_outfilename;
    bool opt_css_infilename;  
    bool opt_xhtml_output;
    bool opt_batch_mode,
         opt_batch_recursive;
    bool opt_rtf_output;
    bool opt_tex_output;
    bool opt_latex_output;
    bool opt_xslfo_output;
    bool opt_fragment;
    bool opt_attach_line_anchors;
    bool opt_show_themes;
    bool opt_show_langdefs;
    bool opt_outdir;
    bool opt_asformat_output;
    bool opt_config_dir,
         opt_data_dir,
         opt_printindex,
         opt_quiet,
         opt_xslfo_fop;

    string helpLang;   
    vector <string> inputFileNames;

    /** l�t Voreinstellungen aus Configdatei .highlight.conf im HOME Verzeichnis
        Kommandozeilenparameter berschreiben diese, falls angegeben */
    void loadPresets();        

    /** gibt Dateiendung zurck, wenn kein "." gefunden wird, wird
        "" zurckgegeben */    
    string getFileSuffix( const string & fileName) const;
        
    void setFormatStyle(const char * );
    
    void readDirectory(const string & wildcard, bool recursive);

  };

#endif

