/***************************************************************************
                          documentstyle.cpp  -  description
                             -------------------
    begin                : Son Nov 10 2002
    copyright            : (C) 2002 by Andr�Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "documentstyle.h"

DocumentStyle::DocumentStyle(const string & styleDefinitionFile)
    : comment(NULL),
    keyword(NULL),
    str(NULL),
    dstr(NULL),
    escapeChar(NULL),
    number(NULL),
    directive(NULL),
    type(NULL),
    line(NULL),
    bgColour(NULL),
    defaultElem(NULL)
{
  fileFound=load(styleDefinitionFile);
}

bool DocumentStyle::load(const string & styleDefinitionFile)
{
  bool fileFound;
  ConfigurationReader styleConfig(DataDir::getThemeDir() + styleDefinitionFile, true);
  if (styleConfig.found())
    {
      fontsize = styleConfig.getParameter("fontsize");
      bgColour = new StyleColour(styleConfig.getParameter("bgcolour"));
      defaultElem  = new ElementStyle(styleConfig.getParameter("defaultcolour"));
      comment = new ElementStyle(styleConfig.getParameter("comment"));
      keyword = new ElementStyle(styleConfig.getParameter("keyword"));
      type = new ElementStyle(styleConfig.getParameter("type"));
      directive = new ElementStyle(styleConfig.getParameter("directive"));
      str = new ElementStyle(styleConfig.getParameter("string"));
      escapeChar = new ElementStyle(styleConfig.getParameter("escapechar"));
      number = new ElementStyle(styleConfig.getParameter("number"));
      dstr = new ElementStyle(styleConfig.getParameter("string_directive"));
      line = new ElementStyle(styleConfig.getParameter("line"));
      fileFound = true;
    }
  else
    {
      fileFound = false;
    }
  return fileFound;
}
//DocumentStyle::DocumentStyle (const DocumentStyle & ds) {}

DocumentStyle::~DocumentStyle()
{
  delete comment;
  delete directive;
  delete escapeChar;
  delete keyword;
  delete number;
  delete str;
  delete dstr;
  delete line;
  delete type;
  delete bgColour;
  delete defaultElem; 
}


string DocumentStyle::getFontSize() const
  {
    return fontsize;
  }
StyleColour* DocumentStyle::getBgColour() const
  {
    return bgColour;
  }
ElementStyle* DocumentStyle::getDefaultStyle() const
  {
    return defaultElem;
  }
ElementStyle* DocumentStyle::getCommentStyle() const
  {
    return comment;
  }
ElementStyle* DocumentStyle::getKeywordStyle() const
  {
    return keyword;
  }
ElementStyle* DocumentStyle::getStringStyle() const
  {
    return str;
  }
ElementStyle* DocumentStyle::getDirectiveStringStyle() const
  {
    return dstr;
  }
ElementStyle* DocumentStyle::getEscapeCharStyle() const
  {
    return escapeChar;
  }
ElementStyle* DocumentStyle::getNumberStyle() const
  {
    return number;
  }
ElementStyle* DocumentStyle::getDirectiveStyle() const
  {
    return directive;
  }
ElementStyle* DocumentStyle::getTypeStyle() const
  {
    return type;
  }
ElementStyle* DocumentStyle::getLineStyle() const
  {
    return line;
  }

bool DocumentStyle::found () const
  {
    return fileFound;
  }
