/***************************************************************************
                          htmlcode.h  -  description
                             -------------------
    begin                : Wed Nov 28 2001
    copyright            : (C) 2001 by André Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef HTMLCODE_H
#define HTMLCODE_H

#include <fstream>  
#include <iostream>
#include <string>
#include <sstream>

#include "version.h"
#include "charcodes.h"
#include "stylecolour.h"
#include "elementstyle.h"

#ifdef USE_WX_LIBRARY

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

// Include wxWindows' headers
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#endif

/**HTMLCode generiert eine CSS/HTMl-Ausgabe mit Syntaxhervorhebung des Sourcecodes.
 
   Class which generates highlighted source code in HTML/CSS.
  *@author Andr�Simon
  */

#include "codeparser.h"

class HtmlCode  : public highlight::CodeParser
  {
  public:
    HtmlCode( highlight::LanguageDefinition &langDef,
              const string &cssStyle,
              const string &cssFilename,
              bool includeCssDef=false,
              bool printLineNumbers=false,
              bool fragment=false,
              bool withAnchors = false,
              bool xhtmlFormat=false,
              int numSpaces=0,
              int wrapStyle=0);
    HtmlCode();
    virtual ~HtmlCode() {};

    /** fuegt am Anfang der neuen Zeile eine fortlaufende Nummer ein
     wenn Nummer nicht 4stellig, wird mit spaceStr rechtsbuendig aufgefuellt.
     Falls -l Option gesetzt, fuege Anchor an Zeilennnummer an */
    virtual void insertLineNumber (enum highlight::State envokingState);


    //Methoden zur Ausgabe des HTML-Codes
    void printHeader(const string &title);
    void printBody();
    void printFooter();

    /** schreibt Style Sheet- Definition in cssFileName */
    bool printCSSFile(const string&, const string&);
  /** schreibt Index-Datei ins Ausgabeverzeichnis */
  bool printIndexFile(const 
                    #ifdef USE_WX_LIBRARY
                      wxListBox* 
                    #else
                      vector<string> &
                    #endif  
                      fileList, const string &outPath );

  private:

    /** Gibt HTML-Code der Sonderzeichen zurueck */
    virtual string maskCharacter(unsigned char );

    /** Kommentar zu den CSS-Definitionen */
    static const string cssComment;

    /** gibt an, ob Anker an Zielenummerierung gesetzt werden sollen*/
    bool attachAnchors;

    /** Pfad zur CSS-Definitionsdatei */
    string cssFile, cssOutDir;

    /** gibt an, ob CSS-Definition in HTML eingebettet wird*/
    bool includeCss;

    /** gibt an, ob eine XHTML-Datei erzeugt werden soll
    falls false, wird HTML Datei erzeugt */
    bool xhtmlOutput;

    /**Liste der altenrnate Stylesheets*/
    //  vector<string> styleList;

    /** gibt CSS String, der die Informationen der Argumente enth�t, zurck*/
    string  formatStyleAttributes(const string & elemName, ElementStyle * elem);

    void  printStyleDefinition(ostream *);
  };

#endif


