/***************************************************************************
                          languagedefinition.h  -  description
                             -------------------
    begin                : Wed Nov 28 2001
    copyright            : (C) 2001 by Andr�Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LANGUAGEDEFINITION_H
#define LANGUAGEDEFINITION_H

#include <string>
#include <set>
#include <map>
#include <iostream>
#include <fstream>
#include <iterator>
#include <sstream>

#include "datadir.h"
#include "configurationreader.h"
#include "stringtools.h"


/**Enthaelt fuer die jeweilige Programmiersprache
   charakteristische Daten.
   Contains specific data of the programming language
   being processed.
 
   @author Andr�Simon
*/
namespace highlight {

class LanguageDefinition
  {

  public:
    LanguageDefinition(string lang);
    LanguageDefinition();

    void printKeys();
    void printTypes();
    string &getSymbolString()
    {
      return symbolString;
    }
    char getSimplifiedStringPrefix(){
      return simplifiedStringPrefix;
    }
    string &getAllowedChars()
    {
      return allowedChars;
    }
    bool getSyntaxHighlight()
    {
      return !disableHighlighting;
    }
    bool isCaseSensitive()
    {
      return caseSensitive;
    }
    bool escCharsOutsideStrings()
    {
      return escCharsAllowedOutsideStrings;
    }

    bool isMatchingStringDelim(const string &,const string &);
    /** gibt true zurueck, wenn s Element von keys ist */
    bool isKeyword(const string &s);
    /** gibt true zurueck, wenn s Element von types ist */
    bool isType(const string &s);

    /** gibt true zurueck, falls c Keyword- oder Variablenprefix */
    bool isPrefix (unsigned char c);
    bool isNumberPrefix (unsigned char c);

    /** l�t die Sprachdefinition */
    bool load(const string& lang);
    /** gibt true zurck, falls Forth Code geparst wird */
    bool iAmForth();

    bool allowNestedMLComments();

    bool highlightingDisabled();
    
    bool typesWithinStringsEnabled();
   
  private:
    // String, der aus Symbolen und den zugehoerigen Symbolkonstnanten
    // zusammengesetzt ist
    string symbolString;

    // String mit Charactern, die in Keywords/Typbezeichnern erlaubt sind
    string allowedChars;

    // Mengen der Schluesselwoerter bzw. Typbezeichnungen
    set
      <string> keys;
    set
      <string> types;
    multimap <string, string> stringCloseDelim;

    bool caseSensitive,
    disableHighlighting,
    escCharsAllowedOutsideStrings,
    // flag, um den Forth Delimiter ")" bei Strings und Kommentaren korrekt zu erkennen
    thisIs4th,
    // Flag, um mehrzeilige Kommentare nicht verschachtelt zu parsen:
    allowNestedMultiLineComments,
    
    enableTypesWithinStrings;

    /** Pfad zur Sprachdefinitionsdatei*/
    string definitionPath;

    // Character, die eine Variable bzw. ein Keyword kennzeichnen
    char variablePrefix,
    keywordPrefix,
    numberPrefix,
    simplifiedStringPrefix;

    /** setzt Membervariablen auf Defaultwerte */
    void reset();
  };

}
#endif
