/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Die Apr 23 22:16:35 CEST 2002
    copyright            : (C) 2002, 2003 by Andr�Simon
    email                : andre.simon1@gmx.de


   Highlight is a universal source code to HTML converter. Syntax highlighting
   is formatted by Cascading Style Sheets. It's possible to easily enhance
   highlight's parsing database.

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "main.h"
 
using namespace std;

//Gibt Versionsinformationen nach cout aus
void HighlightApp::printVersionInfo()
{
  clog << "\n highlight version " 
       << HIGHLIGHT_VERSION
#ifdef USE_PORTABLE_CMDLINE_PARSER
       << "\n (Build with USE_PORTABLE_CMDLINE_PARSER flag set.)"
#endif      
       << "\n Copyright (C) 2002-2004 Andre Simon <andre.simon1@gmx.de>"
       << "\n\n Artistic Style Classes (1.15.3)"
       << "\n Copyright (C) 1998-2002 Tal Davidson <davidsont@bigfoot.com>"
       << "\n\n DirStream Class"
#ifdef USE_PORTABLE_CMDLINE_PARSER
       << ", CommandLineParser Class"
#endif
       << "\n Copyright (C) 2002 Benjamin Kaufmann <hume@c-plusplus.de>"
       << "\n\n This software is released under the terms of the GNU General "
       << "Public License."
       << "\n For more information about these matters, see the file named "
       << "COPYING.\n"
       << endl;
}

void HighlightApp::printBadInstallationInfo()
{
  cerr << "highlight: Data directory not found. Bad installation or wrong "
       << "data-dir parameter."
#ifndef _WIN32
       << "\n\nCopy the highlight files (*.lang, *.style) in one of the "
       << "following directories:\n"
       << "/usr/share/highlight/\n"
       << "/usr/local/share/highlight/\n"
       << "/opt/highlight/\n"
       << "/opt/bin/highlight/\n"
       << "$HOME/.highlight/\n\n"
#else
       << "\n\nCopy the highlight files (*.lang, *.style) in the highlight.exe"
       << " root directory.\n"
#endif
       << "You may also set the data directory with --data-dir.\n";
}

bool HighlightApp::listInstalledFiles(bool themes)
{
  // Liste der Eingabedateien
  vector <string> inFileList;
  unsigned int suffixLength=(themes)?6:5;
  string searchDir = (themes)?DataDir::getThemeDir()+
                     "*.style":DataDir::getLangDefDir()+"*.lang";
  // Liste der Eingabedateien fuellen:
  bool directoryOK = DirectoryReader::getEntries (inFileList,searchDir);
  if (!directoryOK)
    {
     cerr << "highlight: Could not access directory "
          <<  searchDir
          << ", aborted.\n";
      return false;
    }

  clog << "\n  Installed "
       << ((themes)? "themes":"language definitions ")
       << "(located in "
       << ((themes)?DataDir::getThemeDir():DataDir::getLangDefDir())
       << ") :\n"
       << endl;

  string  temp;
  for (unsigned int i=0;i< inFileList.size(); i++)
    {
      if (themes)
       temp = (inFileList[i]).substr(DataDir::getThemeDir().length());
      else
       temp = (inFileList[i]).substr(DataDir::getLangDefDir().length());
      clog << "  "<<temp.substr(0, temp.length()- suffixLength) << endl;
    }
   clog  <<"\n  Use name of the desired "
         << ((themes)?"theme":"language")
         << " with the -"
         << ((themes)?"s":"S")
         << " option.\n" << endl;
   return true;
}

void HighlightApp::printParseError(enum highlight::ParseError err,const string & 
inFile,                      const string & outFile){
  switch (err){
    case highlight::BAD_OUTPUT:
          cerr << "highlight: Could not write to "
               << outFile
               << "."
               << endl;
          break;
     case highlight::BAD_INPUT:
           cerr << "highlight: Input file "
                << inFile
                << " not found."
                << endl;
          break;
     default:;
  }
 }
 
 void HighlightApp::printDebugInfo( highlight::LanguageDefinition &lang, 
                      const string & language)
{    
   clog << "*** DEBUG BEGIN ***\n\n"   
        << "LANGUAGE: " 
        << language
        << "\n\nSYMBOLS: "  
        << lang.getSymbolString()       
        << "\n\nKEYWORDS: "; 
   lang.printKeys();      
   clog << "\n\nTYPES:    ";      
   lang.printTypes(); 
   clog << "\n\n*** DEBUG END ***"<<endl; 
}

/** gibt Dateiendung zurck, wenn kein "." gefunden wird, wird Leerstring ""
    zurckgegeben */
string HighlightApp::getFileSuffix(const string &fileName) 
  {
    unsigned int ptPos=fileName.rfind(".");
    return (ptPos == string::npos) ? 
            "" : fileName.substr(ptPos+1, fileName.length());  
}


/** l�t Liste der bekannten Dateiendungen */
#ifndef _WIN32
bool HighlightApp::loadFileExtensions(){
  ConfigurationReader extensionsConfig(DataDir::getDir() + "extensions.conf", 
true);
#else
bool HighlightApp::loadFileExtensions(const string & path){
  ConfigurationReader extensionsConfig(path + "extensions.conf", true);  
#endif
  
  if (extensionsConfig.found())
  {
    stringstream *values, *names=extensionsConfig.getParameterNames();
    string paramName, paramVal;
    while ( *names >> paramName)
     {
       values = new stringstream 
                (extensionsConfig.getParameter(paramName).c_str()) ;        
        while (*values >> paramVal) {         
          extensions[ paramVal ] = paramName;     
         }
         delete values;
      }
      return true;
    }
  return false;
}

bool HighlightApp::initialize(highlight::LanguageDefinition & lang, 
                              string & suffix, 
                              string & lastSuffix,
                              highlight::CodeParser* parser,
                              CmdLineOptions &options){
  bool langDefinitionFound=false;     
                      
  if (suffix==".lang")   {
       cerr<<"highlight: Undefined language definition. Use -S option."<<endl;
       return false;      
   }

   // Testen, ob neue Sprachdefinition geladen werden muss:
   if (suffix != lastSuffix || !options.enableBatchMode()){
      if (options.printDebugInfo()){
         clog << "Loading new language definition: "
              << suffix<<".lang"
               <<endl;
      }
     langDefinitionFound = lang.load(suffix+".lang");           
     if (!langDefinitionFound) {
       cerr << "highlight: Unknown source file extension \""
            << suffix
            << "\".\n"; 
       return false;
      } else {
       if (options.printDebugInfo()){
          printDebugInfo(lang, suffix);
       }
       lastSuffix = suffix;
       parser->setLanguageDefinition(lang);
       if (options.formattingEnabled()){
         if ((suffix!="c") &&(suffix!="cs")&&(suffix!="java")){
           cerr << "highlight: Code reformatting only enabled with "
                << "C, C++, C# or Java input files.\n";
           parser->setFormatter(DISABLED);
         }
        else{
           parser->setFormatter(options.getFormatStyle());
         }
       }
     }        
  }  
  return true;                            
}


/** gibt bei mehreren gebraeuchlichen Dateiendungen den passenden Namen der
Sprachdefinitionsdatei zurueck */
string HighlightApp::getFileType( string suffix)
{
  if (!extensions[ suffix ].empty())
    {
      return extensions[ suffix ];
    }
  return suffix;
}

int HighlightApp::run(int argc, char**argv){
  
  //get command line options
  CmdLineOptions options(argc, argv);
 
   // set data directory path, where /langDefs and /themes reside
#ifdef _WIN32
  string m_AppPath = argv[0];
	string::size_type Pos = m_AppPath.find_last_of('\\');
	m_AppPath = m_AppPath.substr(0, Pos+1);
  DataDir::searchDataDir((options.newDataDir())?
                                options.getDataDir():m_AppPath );
#else
  // Highlight-Verzeichnis bestimmen
  if (! DataDir::searchDataDir((options.newDataDir())?
                                options.getDataDir():"" ))
    {
      printBadInstallationInfo();
      fatalError=true;
    }
#endif

 // load entries from extensions.conf
  #ifdef _WIN32
  loadFileExtensions(m_AppPath);
  #else
  loadFileExtensions();
  #endif

  if (options.printVersion())
    {
      printVersionInfo();
      return EXIT_SUCCESS;
    }

  if  (options.printHelp())
    {
      Help::printHelp(options.getHelpLang());
      return EXIT_SUCCESS;
    }

  if (options.showThemes() || options.showLangdefs())
    {      
      return listInstalledFiles(options.showThemes())?
               EXIT_SUCCESS:EXIT_FAILURE;
    }
     
  highlight::LanguageDefinition lang;
  highlight::CodeParser  *parser = NULL;

  // get an appropriate code parser instance
  if (options.texFormat())
    {
      parser = new TexCode (lang,
                            options.getStyleName(),
                            options.printLineNumbers(),
                            options.fragmentOutput(),
                            options.getNumberSpaces(),
                            options.getWrappingStyle());
    }
  else if (options.latexFormat())
    {
      parser = new LatexCode(lang,
                             options.getStyleName(),
                             options.printLineNumbers(),
                             options.fragmentOutput(),
                             options.getNumberSpaces(),
                             options.getWrappingStyle());
    }
  else if (options.rtfFormat())
    {
      parser = new RtfCode (lang,
                            options.getStyleName(),
                            options.printLineNumbers(),
                            options.fragmentOutput(),
                            options.getNumberSpaces(),
                            options.getWrappingStyle());
    }
  else if (options.xslfoFormat())
    {
      parser = new XslFoCode(lang,
                             options.getStyleName(),
                             options.printLineNumbers(),
                             options.fragmentOutput(),
                             options.fopCompatible(),
                             options.getNumberSpaces(),
                             options.getWrappingStyle());
    }
  else
    {
      parser = new HtmlCode( lang,
                             options.getStyleName(),
                             options.getCssOutFilename(),
                             options.includeCssDefinitions(),
                             options.printLineNumbers(),
                             options.fragmentOutput(),
                             options.attachLineAnchors(),
                             options.xhtmlFormat(),
                             options.getNumberSpaces(),
                             options.getWrappingStyle());
      // print external css file
      bool cssOk=((HtmlCode*)parser)->printCSSFile(options.getCssInFilename(),
                                                   options.getOutDirectory() + 
                                                   options.getCssOutFilename());
      if (!cssOk){
         cerr << "highlight: Could not write "
              << options.getOutDirectory()
              << options.getCssOutFilename()<<"."<<endl;
      }
    }
  
  // list of input files
  const  vector <string> &inFileList=options.getInputFileNames();
  string outfilePath;
   
  enum highlight::ParseError error;
  unsigned int  size=inFileList.size();
  string suffix, lastSuffix;
  bool exitOnError=false;
    
  if (!options.enableBatchMode()){ 
    //suffix = options.getLanguage();
   if (options.syntaxGiven())
        suffix = options.getLanguage();
   else if (size)
        suffix = getFileType(getFileSuffix(inFileList[0]));   

   exitOnError=!initialize(lang, suffix, lastSuffix, parser,
                             options);  
    if (!exitOnError){
       outfilePath = options.getOutDirectory();
       outfilePath =  options.getSingleOutFilename();
       error = parser->printOutput(options.getSingleInFilename(), outfilePath);
       if (error != highlight::PARSE_OK){
          printParseError(error, options.getSingleInFilename(), outfilePath);
       }
    }
  }
  else {  
    unsigned int i=0;
    
    while (i< size && !exitOnError)
     {      
      suffix = (options.syntaxGiven())?options.getLanguage() :                   
                                                          
getFileType(getFileSuffix(inFileList[i]));                    


      exitOnError=!initialize(lang, suffix, lastSuffix, parser, options);        
      if (exitOnError) break;           
      outfilePath = options.getOutDirectory();      
      if (options.enableBatchMode()){
         string::size_type pos=
             (inFileList[i]).find_last_of(PATH_SEPARATOR_CHAR);    
      
       outfilePath += inFileList[i].substr(pos+1);
       outfilePath +=options.getOutFileSuffix();         

       if (!options.quietMode()) {
           clog << "Writing file "
                << (i+1)
                << " of "
                << size
                << ": "              
                << outfilePath 
                << endl;      
         }
       }
       else {
           outfilePath =  options.getSingleOutFilename();
       }
       error = parser->printOutput(inFileList[i], outfilePath);
       if (error != highlight::PARSE_OK){
         printParseError(error, inFileList[i], outfilePath);
       }
       i++;   
       parser->reset();    
     }
     if (options.printIndexFile()){
         ((HtmlCode*)parser) -> printIndexFile(inFileList, 
                                               options.getOutDirectory());
     }
   }
  delete parser;
  return EXIT_SUCCESS;
}


/*********************************************/ 

int main(int argc, char **argv)
{
  HighlightApp app;
  return app.run(argc, argv);
}


