/***************************************************************************
                          rtfcode.cpp  -  description
                             -------------------
    begin                : Die Jul 9 2002
    copyright            : (C) 2002 by Andr�Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rtfcode.h"

using namespace std;
using namespace highlight;

string  RtfCode::formatStyleAttributes( ElementStyle * col)
{
  stringstream s;
  s  << "\\red"<< col->getColour()->getRTFRedValue()
  << "\\green"<<col->getColour()->getRTFGreenValue()
  << "\\blue"<<col->getColour()->getRTFBlueValue()
  << ";";

  return s.str();
}

string  RtfCode::getOpenTag(int styleNumber,ElementStyle * elem)
{
  stringstream s;
  s << "{\\cf"<<(styleNumber+1)<<"{";  
  s << ( elem->isBold() ?"\\b " :"" )
    << ( elem->isItalic() ?"\\i " :"" )
    << ( elem->isUnderline() ?"\\ul " :"" );  
  return  s.str();
}

string  RtfCode::getCloseTag(ElementStyle * elem)
{
  stringstream s;
  s << ( elem->isBold() ?"\\b0 " :"" )
    << ( elem->isItalic() ?"\\i0 " :"" )
    << ( elem->isUnderline() ?"\\ul0 " :"" )
    << "}}";
  return  s.str();
}



// Definition der RTF-Farbcodes, die zum Highlighting benutzt werden:

RtfCode::RtfCode( LanguageDefinition &langDef,const string &cssStyle,
                  bool printLineNumbers, bool fragment, int numSpaces, int wrapStyle)
    : CodeParser(langDef, printLineNumbers, cssStyle, numSpaces, wrapStyle, fragment)
{

if (docStyle->found()){

  styleTagOpen[0] =getOpenTag(STANDARD, docStyle->getDefaultStyle());
  styleTagOpen[1] =getOpenTag(KEYWORD, docStyle->getKeywordStyle());
  styleTagOpen[2] =getOpenTag(TYPE, docStyle->getTypeStyle());
  styleTagOpen[3] =getOpenTag(STRING, docStyle->getStringStyle());
  styleTagOpen[4] =getOpenTag(NUMBER, docStyle->getNumberStyle());
  styleTagOpen[5] =getOpenTag(SINGLE_LINE_COMMENT, docStyle->getCommentStyle());
  styleTagOpen[6] =getOpenTag(MULTI_LINE_COMMENT_BEGIN, docStyle->getCommentStyle());
  styleTagOpen[7] =getOpenTag(ESC_CHAR, docStyle->getEscapeCharStyle());
  styleTagOpen[8] =getOpenTag(DIRECTIVE_LINE, docStyle->getDirectiveStyle());
  styleTagOpen[9] =getOpenTag(DIRECTIVE_STRING, docStyle->getDirectiveStringStyle());
  styleTagOpen[10] =getOpenTag(LINENUMBER, docStyle->getLineStyle());

  styleTagClose[0] =getCloseTag(docStyle->getDefaultStyle());
  styleTagClose[1] =getCloseTag( docStyle->getKeywordStyle());
  styleTagClose[2] =getCloseTag( docStyle->getTypeStyle());
  styleTagClose[3] =getCloseTag(docStyle->getStringStyle());
  styleTagClose[4] =getCloseTag( docStyle->getNumberStyle());
  styleTagClose[5] =getCloseTag( docStyle->getCommentStyle());
  styleTagClose[6] =getCloseTag( docStyle->getCommentStyle());
  styleTagClose[7] =getCloseTag( docStyle->getEscapeCharStyle());
  styleTagClose[8] =getCloseTag( docStyle->getDirectiveStyle());
  styleTagClose[9] =getCloseTag(docStyle->getDirectiveStringStyle());
  styleTagClose[10] =getCloseTag( docStyle->getLineStyle());

  stringstream s;
  s << formatStyleAttributes(docStyle->getDefaultStyle());
  s << formatStyleAttributes( docStyle->getKeywordStyle());
  s << formatStyleAttributes(docStyle->getTypeStyle());
  s << formatStyleAttributes(docStyle->getStringStyle());
  s << formatStyleAttributes(docStyle->getNumberStyle());
  s << formatStyleAttributes(docStyle->getCommentStyle());
  s << formatStyleAttributes(docStyle->getCommentStyle());
  s << formatStyleAttributes(docStyle->getEscapeCharStyle());
  s << formatStyleAttributes(docStyle->getDirectiveStyle());
  s << formatStyleAttributes(docStyle->getStringStyle());
  s << formatStyleAttributes(docStyle->getDirectiveStringStyle());
  s << formatStyleAttributes(docStyle->getLineStyle());
  styleDefinition = s.str();
}
  newLineTag = "\\par\\pard\n";
  spacer = " ";
}

RtfCode::RtfCode()
{}
RtfCode::~RtfCode()
{}

void RtfCode::printHeader(const string & title)
{

  *out << "{\\rtf1\\ansi\\uc0 \\deff1"
  << "{\\fonttbl{\\f1\\fmodern\\fprq1\\fcharset0 Courier;}}"
  << "{\\colortbl;";
  *out <<  styleDefinition;
  *out << "}\n{\\info }\\paperw11900\\paperh16820\\margl560\\margr560\\margt840\\margb840\\widowctrl\\ftnbj\\aenddoc\\formshade \\fet0\\sectd"
  << "\\linex0\\endnhere\\plain\\f1\\fs20\n\\pard "
  << flush;
}

/** gibt RTF-Text aus */
void RtfCode::printBody()
{
  processStandardState();
}


void RtfCode::printFooter()
{
  *out << "}}"<<endl;
}

/** Gibt RTF-Code der Sonderzeichen zurueck */
string RtfCode::maskCharacter(unsigned char c)
{
  string m;
  switch (c)
    {
    case '}' :
      return "\\}";
      break;
    case '{' :
      return "\\{";
      break;
    case '\\' :
      return "\\\\";
      break;
    case '0':
      return "{0}";
      break;
    case '1':
      return "{1}";
      break;
    case '2':
      return "{2}";
      break;
    case '3':
      return "{3}";
      break;
    case '4':
      return "{4}";
      break;
    case '5':
      return "{5}";
      break;
    case '6':
      return "{6}";
      break;
    case '7':
      return "{7}";
      break;
    case '8':
      return "{8}";
      break;
    case '9':
      return "{9}";
      break;
    case AUML_LC:
      return "\\'e4";
      break;
    case OUML_LC:
      return "\\'f6";
      break;
    case UUML_LC:
      return "\\'fc";
      break;
    case AUML_UC:
      return "\\'c4";
      break;
    case OUML_UC:
      return "\\'d6";
      break;
    case UUML_UC:
      return "\\'dc";
      break;
      
    case AACUTE_LC:
      return "\\'e1";
      break;
    case EACUTE_LC:
      return "\\'e9";
      break;    
    case OACUTE_LC:
      return "\\'f3";
      break;
    case UACUTE_LC:
      return "\\'fa";
      break;
      
    case AGRAVE_LC:
      return "\\'e0";
      break;
    case EGRAVE_LC:
      return "\\'e8";
      break;    
    case OGRAVE_LC:
      return "\\'f2";
      break;
    case UGRAVE_LC:
      return "\\'f9";
      break;
      
    case AACUTE_UC:
      return "\\'c1";
      break;
    case EACUTE_UC:
      return "\\'c9";
      break;
    case OACUTE_UC:
      return "\\'d3";
      break;
    case UACUTE_UC:
      return "\\'da";
      break;
    case AGRAVE_UC:
      return "\\'c0";
      break;
    case EGRAVE_UC:
      return "\\'c8";
      break;    
    case OGRAVE_UC:
      return "\\'d2";
      break;
    case UGRAVE_UC:
      return "\\'d9";
      break;
      
    case SZLIG:
      return "\\'df";
      break;
   // skip  first byte of multibyte chracters
    #ifndef _WIN32
    case 195:
      return string("");
      break; 
    #endif

    default :
      m += c;
      return m;
    }
}
