/***************************************************************************
                          TexCode.cpp  -  description
                             -------------------
    begin                : Mit Jul 24 2002
    copyright            : (C) 2002 by Andr�Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include "texcode.h"

using namespace highlight;

TexCode::TexCode(highlight::LanguageDefinition &langDef,
                 const string &cssStyle,
                 bool printLineNumbers, bool fragment, 
                 int numSpaces, int wrapStyle): 
  CodeParser(langDef, printLineNumbers, cssStyle, numSpaces, 
             wrapStyle, fragment) { 

  if (!docStyle->found()) return;

  styleTagOpen[STANDARD] = "{\\hlstd ";
  styleTagOpen[KEYWORD] = "{\\hlkey ";
  styleTagOpen[TYPE] = "{\\hltyp ";
  styleTagOpen[STRING] = "{\\hlstr ";
  styleTagOpen[NUMBER] = "{\\hlnum ";
  styleTagOpen[SINGLE_LINE_COMMENT] = "{\\hlcom ";
  styleTagOpen[MULTI_LINE_COMMENT_BEGIN] = "{\\hlcom ";
  styleTagOpen[ESC_CHAR] = "{\\hlesc ";
  styleTagOpen[DIRECTIVE_LINE] = "{\\hldir ";
  styleTagOpen[DIRECTIVE_STRING] = "{\\hldstr ";
  styleTagOpen[LINENUMBER] = "{\\hlline ";
  for (int i=0;i<NUMBER_OF_STYLES; i++)    {      
    styleTagClose[i] = "}";
  }
  newLineTag = "\\par\n";
  spacer = "\\ ";

  maskWs=true;
  maskWsBegin = "{\\rm\\hskip ";
  maskWsEnd = " em}";  
}

TexCode::TexCode()
{}
TexCode::~TexCode()
{}


void TexCode::printHeader(const string & title)
{
  *out << formatStyleAttributes("std", docStyle->getDefaultStyle());
  *out << formatStyleAttributes("key", docStyle->getKeywordStyle());
  *out << formatStyleAttributes("num", docStyle->getNumberStyle());
  *out << formatStyleAttributes("typ", docStyle->getTypeStyle());
  *out << formatStyleAttributes("esc", docStyle->getEscapeCharStyle());
  *out << formatStyleAttributes("str", docStyle->getStringStyle());
  *out << formatStyleAttributes("dstr", docStyle->getDirectiveStringStyle());
  *out << formatStyleAttributes("com", docStyle->getCommentStyle());
  *out << formatStyleAttributes("dir", docStyle->getDirectiveStyle());
  *out << formatStyleAttributes("line", docStyle->getLineStyle());
  *out << "\\nopagenumbers"
       << "\\input colordvi \\tt\n"
       <<  "% The special option is not supported by all dvi drivers\n"
       << "\\special{background rgb ";
 // if (docStyle->found()) { wird in Codeparser::printOutput abgefangen
  *out << docStyle->getBgColour() ->getLatexRedValue() << " "
       << docStyle->getBgColour()->getLatexGreenValue() << " "
       << docStyle->getBgColour()->getLatexBlueValue() << "}"  
       << endl;
  //}
}

string TexCode::formatStyleAttributes(const string & elemName,ElementStyle * elem)
{
  stringstream s;
  s << "\\def\\hl"
    << elemName 
    << "{";
  if (elem->isBold())
    s << "\\bf";
  if (elem->isItalic())
    s << "\\it";  
  s  <<  "\\textColor{"
     << (elem->getColour()->getTexRedValue())<<" "
     << (elem->getColour()->getTexGreenValue())<<" "
     << (elem->getColour()->getTexBlueValue())<<" 0.0}}\n";
  return  s.str();
}

/** Gibt Tex-Code der Sonderzeichen zurueck */
string TexCode:: maskCharacter(unsigned char c)
{
  string m;

  switch (c)
    {
    case '{':
    case '}':
      m += "$\\";
      m += c;
      m += "$";
      return m;
      break;
    case '^':
    case '_':
      m += "\\";
      m += c;
      m += "{}";
      return m;
      break;
    case '&':
    case '$':
    case '#':
    case '%':
      m+="\\";
      m += c;
      return m;
      break;
    case '\\':
      return "$\\backslash$";
      break;
    case ' ':
      return "\\ ";
      break;
    case AUML_LC:
      return "\\\"a";
      break;
    case OUML_LC:
      return "\\\"o";
      break;
    case UUML_LC:
      return "\\\"u";
      break;
    case AUML_UC:
      return "\\\"A";
      break;
    case OUML_UC:
      return "\\\"O";
      break;
    case UUML_UC:
      return "\\\"U";
      break;
    case AACUTE_LC:
      return "\\'a";
      break;
    case EACUTE_LC:
      return "\\'e";
      break;    
    case OACUTE_LC:
      return "\\'o";
      break;
    case UACUTE_LC:
      return "\\'u";
      break;
    case AGRAVE_LC:
      return "\\`a";
      break;
    case EGRAVE_LC:
      return "\\`e";
      break;    
    case OGRAVE_LC:
      return "\\`o";
      break;
    case UGRAVE_LC:
      return "\\`u";
      break;
    case AACUTE_UC:
      return "\\'A";
      break;
    case EACUTE_UC:
      return "\\'E";
      break;    
    case OACUTE_UC:
      return "\\'O";
      break;
    case UACUTE_UC:
      return "\\'U";
      break;
    case AGRAVE_UC:
      return "\\`A";
      break;
    case EGRAVE_UC:
      return "\\`E";
      break;    
    case UGRAVE_UC:
      return "\\`O";
      break;
    case OGRAVE_UC:
      return "\\`U";
      break;
    case SZLIG:
      return "\\ss ";
      break;
     #ifndef _WIN32
 // skip  first byte of multibyte chracters
    case 195:
      return string("");
      break;    
     #endif   
  
    default :
      string m;
      m += c;
      return m;
    }

}
/** gibt Tex-Text aus */
void TexCode::printBody()
{
  processStandardState();
}

void TexCode::printFooter()
{
  *out << "\\bye\n"
       << "% TeX generated by Highlight "
       << HIGHLIGHT_VERSION
       << ", "
       << HIGHLIGHT_URL
       << endl;
}



